﻿local playerName = UnitName("player")
local playerGUID = UnitGUID("player")
local DeathsTable = {}
local KillsTable = {}

local LastKill = nil
local Kills = 0
local Streak = 0

local PlayedIndex = 0
local ButtonText = PVPReadyDialogEnterBattleButton:GetText() .. " "

local function Play(file)
	PlaySoundFile(file, "Master")
	PlaySoundFile(file, "SFX")
end

local function Emote(str)
	SendChatMessage(str, "EMOTE")
end

PVPReadyDialogLeaveQueueButton:Hide()
PVPReadyDialogLeaveQueueButton.Show = function() end
PVPReadyDialog:HookScript("OnUpdate", function(self)
	if self.activeIndex then
		PVPReadyDialogEnterBattleButton:SetText(ButtonText .. GetBattlefieldPortExpiration(self.activeIndex))
	end
end)

hooksecurefunc("PVPReadyDialog_Update", function(self, index)
	local status = GetBattlefieldStatus(index);
	if PlayedIndex == 0 and status == "confirm" then
		PlayedIndex = index
		Play("Interface/AddOns/Zadrot_KillSound/Timer/Play.ogg")
	elseif PlayedIndex == index and (status == "queued" or status == "active" or status == "none") then
		PlayedIndex = 0
	end
end)

local FILTER_MY_PETS = bit.bor(COMBATLOG_OBJECT_TYPE_OBJECT, COMBATLOG_OBJECT_TYPE_GUARDIAN, COMBATLOG_OBJECT_TYPE_PET, COMBATLOG_OBJECT_CONTROL_PLAYER, COMBATLOG_OBJECT_REACTION_FRIENDLY,  COMBATLOG_OBJECT_AFFILIATION_MINE)
local FILTER_ENEMY_PLAYERS = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_CONTROL_PLAYER, COMBATLOG_OBJECT_REACTION_MASK, COMBATLOG_OBJECT_AFFILIATION_MASK)

local function OnEvent(self, event)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local timestamp, type, _,  sourceGUID, sourceName, sourceFlags, _, destGUID, destName, destFlags, _, _, swingOverkill, _, _, spellOverkill = CombatLogGetCurrentEventInfo()
		
		local ToEnemy
		local FromEnemy
		local FromMyPets

		if destName and not CombatLog_Object_IsA(destFlags, COMBATLOG_OBJECT_NONE) then
			ToEnemy = CombatLog_Object_IsA(destFlags, FILTER_ENEMY_PLAYERS)
		end

		if sourceName and not CombatLog_Object_IsA(sourceFlags, COMBATLOG_OBJECT_NONE) then
			FromMyPets = CombatLog_Object_IsA(sourceFlags, FILTER_MY_PETS)
			FromEnemy = CombatLog_Object_IsA(sourceFlags, FILTER_ENEMY_PLAYERS)
		end
			
		if (type == "PARTY_KILL" and sourceGUID == playerGUID and ToEnemy)
		or (type == "SWING_DAMAGE" and destGUID ~= playerGUID and FromMyPets and ToEnemy and swingOverkill >= 0)
		or ((type == "RANGE_DAMAGE" or type == "SPELL_DAMAGE" or type == "SPELL_PERIODIC_DAMAGE") and destGUID ~= playerGUID and FromMyPets and ToEnemy and spellOverkill >= 0) then

			if (KillsTable[destName] and (timestamp - KillsTable[destName]) < 5) then
				return
			else
				KillsTable[destName] = timestamp
			end

			if LastKill and (timestamp - LastKill < 30) then
				Streak = Streak + 1
			else
				Streak = 1
				Kills = Kills + 1
			end

			if Streak == 2 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/DoubleKill.ogg")
				Emote("Double Kill")
			elseif Streak == 3 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/MultiKill.ogg")
				Emote("Multi Kill")
			elseif Streak == 4 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/MegaKill.ogg")
				Emote("Mega Kill")
			elseif Streak == 5 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/UltraKill.ogg")
				Emote("Ultra Kill")
			elseif Streak == 6 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/MonsterKill.ogg")
				Emote("Monster Kill")
			elseif Streak == 7 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/LudicrousKill.ogg")
				Emote("Ludicrous Kill")
			elseif Streak >= 8 then
				Play("Interface/AddOns/Zadrot_KillSound/Multikill/HolyShit.ogg")
				Emote("Holy Shit")

			elseif Streak <= 1 then
				if (DeathsTable[destName] and (timestamp - DeathsTable[destName]) < 90) then
					DeathsTable[destName] = nil
					Play("Interface/AddOns/Zadrot_KillSound/Revenge/Retribution.ogg")
					Emote("Retribution")
				elseif Kills == 1 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/FirstBlood.ogg")
					Emote("First Blood")
				elseif Kills == 2 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/KillingSpree.ogg")
					Emote("Killing Spree")
				elseif Kills == 3 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Rampage.ogg")
					Emote("Rampage")
				elseif Kills == 4 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Dominating.ogg")
					Emote("Dominating")
				elseif Kills == 5 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Unstoppable.ogg")
					Emote("Unstoppable")
				elseif Kills == 6 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/GodLike.ogg")
					Emote("GodLike")
				elseif Kills == 7 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/WickedSick.ogg")
					Emote("Wicked Sick")
				elseif Kills == 8 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Berzerk.ogg")
					Emote("Berzerk")
				elseif Kills == 9 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Headhunter.ogg")
					Emote("Headhunter")
				elseif Kills == 10 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Slaughter.ogg")
					Emote("Slaughter")
				elseif Kills == 11 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Termination.ogg")
					Emote("Termination")
				elseif Kills == 12 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Erradication.ogg")
					Emote("Erradication")
				elseif Kills == 13 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Annihilation.ogg")
					Emote("Annihilation")
				elseif Kills == 14 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Extermination.ogg")
					Emote("Extermination")
				elseif Kills == 15 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Bloodbath.ogg")
					Emote("Bloodbath")
				elseif Kills >= 16 then
					Play("Interface/AddOns/Zadrot_KillSound/Kill/Ownage.ogg")
					Emote("Ownage")
				end
			end

			LastKill = timestamp

		elseif (type == "SWING_DAMAGE" and FromEnemy and destGUID == playerGUID and swingOverkill >= 0)
		or ((type == "RANGE_DAMAGE" or type == "SPELL_DAMAGE" or type == "SPELL_PERIODIC_DAMAGE") and FromEnemy and destGUID == playerGUID and spellOverkill>= 0) then
			if sourceName ~= nil and sourceName ~= playerName then

				if (DeathsTable[sourceName] and (timestamp - DeathsTable[sourceName]) < 5) then
					return
				else
					DeathsTable[sourceName] = timestamp
				end

				if KillsTable[sourceName] and (timestamp - KillsTable[sourceName]) < 90 then
					KillsTable[sourceName] = nil
					Play("Interface/AddOns/Zadrot_KillSound/Revenge/Denied.ogg")
					Emote("Denied")
				end
			end
		end

	elseif event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_DEAD" then
		LastKill = nil
		Kills = 0
		Streak = 0
	end
end

local f = CreateFrame("Frame")
f:SetScript("OnEvent", OnEvent)
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:RegisterEvent("ZONE_CHANGED_NEW_AREA")
f:RegisterEvent("PLAYER_DEAD")